@@ -1,9 +1,10 @@
 import yaml
 
-def load_yaml(filename)->dict:
+
+def load_yaml(filename) -> dict:
     with open(filename) as f:
-        settings = yaml.load(f, Loader=yaml.FullLoader)
-    return settings
+        return yaml.load(f, Loader=yaml.FullLoader)
+
 
 def save_yaml(filename, settings):
     with open(filename, "w") as file:
